#coding: utf-8

import base64
import os
from tempfile import NamedTemporaryFile

from lxml import etree

from helpers import run_cmd, tags, _from_soap
from skeleton import make_node_with_ns
from namespaces import NS_MAP


class SignerError(Exception):
    pass


def _format_pem(data):
    u'''
    Форматирование текстового представления сертификата
    на линии по 64 символа и заключение его в теги для
    передачи OpenSSL.

    @param  data   Текст сертификата, закодированный в base64.
    @type   unicode

    @returns PEM, содержащий отформатированный сертификат.
    @rtype    unicode
    '''

    result = []

    data = data.replace('\n', '')
    brackets = range(0, len(data) + 64, 64)

    result.append('-----BEGIN CERTIFICATE-----')
    for start, end in zip(brackets, brackets[1:]):
        result.append(data[start:end])
    result.append('-----END CERTIFICATE-----')

    return '\n'.join(result)


def load_cert_from_pem(data):
    u'''
    Загрузка данных публичного сертификата из PEM-контейнера
    (RFC 1421-1424).

    @param  pem_filename    Имя файла PEM-контейнера.
    @type   pem_filename    unicode

    @return base64-представление данных сертификата.
    @rtype  unicode
    '''

    assert data, 'No PEM provided!'

    data = data.replace('\r', '').replace('\n', '')
    cert_start = data.find('-----BEGIN CERTIFICATE-----')
    cert_end = data.find('-----END CERTIFICATE-----')

    # Не найдены маркеры начала и окончания сертификата в PEM
    if not (cert_start > -1 and cert_end > -1):
        raise SignerError('PEM has no certificate markers (BEGIN, END)!')

    return data[cert_start + 27:cert_end]


def load_pubkey_from_pem(data):
    u'''
    Загрузка данных публичного ключа из PEM-контейнера
    (RFC 1421-1424).

    @param  pem_filename    Имя файла PEM-контейнера.
    @type   pem_filename    unicode

    @return base64-представление данных публичного ключа.
    @rtype  unicode
    '''

    assert data, 'No PEM provided!'

    load_key_cmd = ['openssl', 'x509', '-inform', 'PEM', '-pubkey', '-noout']

    out, err = run_cmd(load_key_cmd, input=data)

    if err:
        raise SignerError(unicode(err))

    return out


def c14n_tags(tag):
    u'''
    Исключительная каноникализация (см. http://www.w3.org/TR/xml-exc-c14n/)
    дерева XML-элементов.

    @param  tag     Корень дерева XML-элементов.
    @type   tag     lxml.Element

    @return Строковое представление каноникализированной формы XML-дерева.
    @rtype  unicode
    '''

    return etree.tostring(tag, method='c14n', exclusive=True, with_comments=False)


def get_text_signature(text, private_key_fn, private_key_pass):
    u'''
    Получение ЭП указанного текста через вызов внешнего экземпляра OpenSSL,
    с использованием частного ключа ОИВ.

    @param  text    Подписываемый текст.
    @type   text    unicode

    @param  private_key_fn  Путь к PEM-файлу, содержащему частный ключ.
    @param  private_key_fn  unicode

    @param  private_key_pass    Пароль к частному ключу.
    @type   private_key_pass    unicode

    @return Закодированная в base64 ЭП текста.
    @rtype  unicode
    '''
    openssl_sign_cmd = [
        'openssl', 'dgst', '-sign', private_key_fn, '-binary',
        '-md_gost94', '-passin', 'stdin']

    out, err = run_cmd(openssl_sign_cmd, input=private_key_pass + '\n' + text)
    if err:
        raise ValueError(u'OpenSSL error: %s' % err)

    return base64.b64encode(out)


def get_text_digest(text):
    u'''
    Получение текстового представления хэш-кода переданного текста
    по ГОСТ Р 34.11-94.

    @param  text    Текст, хэш-код которого необходимо получить.
    @type   text    unicode

    @return Закодированный в base64 хэш-код текста.
    @rtype  unicode
    '''
    openssl_sign_cmd = ['openssl', 'dgst', '-binary', '-md_gost94']

    out, err = run_cmd(openssl_sign_cmd, input=text)
    if err:
        raise ValueError(u'OpenSSL error: %s' % err)

    return base64.b64encode(out)


def get_file_digest(fn):
    u'''
    Получение текстового представления хэш-кода переданного файла
    по ГОСТ Р 34.11-94.

    @param  fn    Путь к файлу, хэш-код которого необходимо получить.
    @type   fn    unicode

    @return Закодированный в base64 хэш-код текста.
    @rtype  unicode
    '''
    openssl_sign_cmd = ['openssl', 'dgst', '-binary', '-md_gost94', fn]

    out, err = run_cmd(openssl_sign_cmd)
    if err:
        raise ValueError(u'OpenSSL error: %s' % err)

    return base64.b64encode(out)


def construct_wsse_header(digest=None, signature=None, certificate=None):
    u'''
    Формирование в виде дерева XML-элементов заголовка WS-Security.

    @param  digest  Хэш-код подписи элементов сообщения.
    @type   digest  unicode

    @param  signature ЭП сообщения.
    @type   signature unicode

    @param  certificate     Открытый ключ сообщения.
    @type   certificate     unicode

    @return WS-Security заголовок.
    @rtype  lxml.Element
    '''
    ds_node = make_node_with_ns('ds')
    wsse_node = make_node_with_ns('wsse')

    security_node = wsse_node('Security')
    security_node.attrib['{%s}actor' % NS_MAP['SOAP-ENV']] = 'http://smev.gosuslugi.ru/actors/smev'

    binary_sec_token_node = wsse_node('BinarySecurityToken')

    signature_node = ds_node('Signature')
    signed_info_node = ds_node('SignedInfo')
    c14n_method_node = ds_node('CanonicalizationMethod')
    signature_method_node = ds_node('SignatureMethod')
    reference_node = ds_node('Reference')
    transforms_node = ds_node('Transforms')
    transform1_node = ds_node('Transform')
    transform2_node = ds_node('Transform')
    key_info_node = ds_node('KeyInfo')
    digest_method_node = ds_node('DigestMethod')
    digest_value_node = ds_node('DigestValue')
    signature_value_node = ds_node('SignatureValue')

    sec_token_reference_node = wsse_node('SecurityTokenReference')
    token_reference_node = wsse_node('Reference')

    # FIX: Они вообще здесь нужны? Метод. рекомендации сами себе противоречат :-\
    #x509_data_node = ds_node('X509Data')
    #x509_cert_node = ds_node('X509Certificate')

    binary_sec_token_node.text = certificate

    # Установка предопределенных значений согласно метод. рекомендациям v. 2.5.6
    c14n_method_node.attrib['Algorithm'] = 'http://www.w3.org/2001/10/xml-exc-c14n#'
    transform2_node.attrib['Algorithm'] = 'http://www.w3.org/2001/10/xml-exc-c14n#'
    transform1_node.attrib['Algorithm'] = 'http://www.w3.org/2000/09/xmldsig#enveloped-signature'
    digest_method_node.attrib['Algorithm'] = 'http://www.w3.org/2001/04/xmldsig-more#gostr3411'
    signature_method_node.attrib['Algorithm'] = 'http://www.w3.org/2001/04/xmldsig-more#gostr34102001-gostr3411'

    reference_node.attrib['URI'] = '#body'
    binary_sec_token_node.attrib['EncodingType'] = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary'
    binary_sec_token_node.attrib['ValueType'] = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3'
    binary_sec_token_node.attrib['{%s}Id' % NS_MAP['wsu']] = 'CertId'
    token_reference_node.attrib['URI'] = '#CertId'
    token_reference_node.attrib['ValueType'] = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-x509-token-profile-1.0#X509v3'

    sec_token_reference_node.append(token_reference_node)

    key_info_node.extend([
        sec_token_reference_node])
        #x509_data_node,
        #x509_cert_node])

    transforms_node.extend([
        transform1_node,
        transform2_node])

    reference_node.extend([
        transforms_node,
        digest_method_node,
        digest_value_node])

    signed_info_node.extend([
        c14n_method_node,
        signature_method_node,
        reference_node])

    signature_node.extend([
        signed_info_node,
        signature_value_node,
        key_info_node])

    security_node.extend([
        binary_sec_token_node,
        signature_node])

    return security_node


def sign_document(doc, priv_key_fn, priv_key_pass):
    u'''
    Подписание сообщения без вложения согласно ГОСТ Р 34.10-2001.

    @param  doc     Подписываемый XML-документ, содержащий себе в себе СМЭВ-сообщение.
    @type   doc     lxml.Element

    @param  priv_key_fn     Путь к файлу с частному ключу подписи.
    @type   priv_key_fn     unicode

    @param  priv_key_pass   Пароль к частному ключу подписи.
    @type   priv_key_pass   unicode

    @return Подписанный XML-документ.
    @rtype  lxml.Element
    '''
    header_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Header')
    security_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Header/wsse:Security')

    if not security_node:
        with open(priv_key_fn, 'rb') as priv_key_file:
            cert_data = load_cert_from_pem(priv_key_file.read())
        wsse_header_node = construct_wsse_header(certificate=cert_data)
        header_node[0].append(wsse_header_node)

    body_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Body')
    c14n_body = c14n_tags(body_node[0])
    digest_value = get_text_digest(c14n_body)

    digest_val_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Header/wsse:Security/ds:Signature/ds:SignedInfo/ds:Reference/ds:DigestValue')
    digest_val_node[0].text = digest_value

    sign_info_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Header/wsse:Security/ds:Signature/ds:SignedInfo')
    c14n_sign_info = c14n_tags(sign_info_node[0])
    signature_value = get_text_signature(c14n_sign_info, priv_key_fn, priv_key_pass)

    sign_val_node = tags(doc, '/SOAP-ENV:Envelope/SOAP-ENV:Header/wsse:Security/ds:Signature/ds:SignatureValue')
    sign_val_node[0].text = signature_value

    return doc


def verify_gost94_signature(text, public_key, signature_value):
    u'''
    Проверка корректности ЭП переданного текста по ГОСТ Р 34.11-94.

    @param  text    Текст, подпись которого проверяется.
    @type   text    unicode

    @param  public_key  Публичный ключ, которым подписывался текст.
    @type   public_key  unicode

    @param  signature_value Подпись.
    @type   signature_value unicode

    @return Флаг корректности ЭП текста.
    @type   boolean
    '''

    # Так как OpenSSL не умеет считывать значения подписи и проверяемый
    # текст со стандартного ввода, мы вынуждены использовать временные файлы.
    # После записи они закрываются, т.к. Windows не позволяет считывать
    # открытые файлы.
    tmp_public_key = NamedTemporaryFile(delete=False)
    tmp_public_key.file.write(public_key)
    tmp_public_key.file.close()

    tmp_signature_value = NamedTemporaryFile(delete=False)
    tmp_signature_value.file.write(base64.b64decode(signature_value))
    tmp_signature_value.file.close()

    openssl_sign_cmd = ['openssl', 'dgst', '-md_gost94', '-verify',
                        tmp_public_key.name, '-signature',
                        tmp_signature_value.name]

    out, err = run_cmd(openssl_sign_cmd, input=text)

    if err:
        raise SignerError(unicode(err))

    # Убираем за собой
    os.remove(tmp_signature_value.name)
    os.remove(tmp_public_key.name)

    return out.strip() == "Verified OK"


def verify_envelope_signature(envelope):
    u'''
    Проверка подписи SOAP-запроса по ГОСТ Р 34.11-94.

    @param envelope     Подписанный XML-документ.
    @type  envelope     lxml.Element

    @return Флаг корректности подписи документа.
    @rtype  boolean
    '''

    header, body = _from_soap(envelope)

    if body is None:
        raise SignerError("'Body' tag not found in SOAP envelope!'")

    digest_value = tags(envelope, ".//ds:DigestValue")
    if not digest_value:
        raise SignerError("'DigestValue' tag is not found")

    binary_security_token = tags(envelope, './/wsse:BinarySecurityToken')
    if not binary_security_token:
        raise SignerError("'BinarySecurityToken' tag is not found")

    signed_info = tags(envelope, './/ds:SignedInfo')
    if not signed_info:
        raise SignerError("`SignedInfo' tag is not found")

    signature_value = tags(envelope, ".//ds:SignatureValue")
    if not signed_info:
        raise SignerError("`SignatureValue' tag is not found")

    if digest_value[0].text != get_text_digest(c14n_tags(body)):
        return False

    # Извлекаем публичный ключ из заголовка WS-Security
    tmp_cert = _format_pem(binary_security_token[0].text)
    public_key = load_pubkey_from_pem(tmp_cert)

    return verify_gost94_signature(c14n_tags(signed_info[0]), public_key,
                                   signature_value[0].text)



#coding: utf-8

import shutil
import base64
import StringIO
import uuid
import unittest
import os
import zipfile

from lxml import etree
from mimetypes import types_map
from tempfile import NamedTemporaryFile, mkdtemp

from skeleton import construct_smev_envelope
from helpers import dict_to_xmldoc
from namespaces import NS_MAP
from signer import sign_document, verify_envelope_signature, get_text_digest
from attachments import encode_directory, extract_directory

# Тестовый ключ
PEM = r'''
-----BEGIN CERTIFICATE-----
MIIBtzCCAWQCCQCUhYafJf4zaTAKBgYqhQMCAgMFADBiMQswCQYDVQQGEwJSVTES
MBAGA1UECAwJVGF0YXJzdGFuMQ4wDAYDVQQHDAVLYXphbjETMBEGA1UECgwKQkFS
Uy1Hcm91cDEMMAoGA1UECwwDRURVMQwwCgYDVQQDDANFRFUwHhcNMTIxMDIxMTAz
MzQyWhcNMTMxMDIyMTAzMzQyWjBiMQswCQYDVQQGEwJSVTESMBAGA1UECAwJVGF0
YXJzdGFuMQ4wDAYDVQQHDAVLYXphbjETMBEGA1UECgwKQkFSUy1Hcm91cDEMMAoG
A1UECwwDRURVMQwwCgYDVQQDDANFRFUwYzAcBgYqhQMCAhMwEgYHKoUDAgIjAQYH
KoUDAgIeAQNDAARAxdw/hP6gGRCqSYKMptrnObSGeu6OZ1Cc80WmbPR7S34hSvjG
Ie2fEClMpDzBKZjuMsIijY3QxlIWyLZXYMX3bzAKBgYqhQMCAgMFAANBAMc+WaiP
p8PLXZHAL6EgXDXomPIA7s97dHvPYqOlfuAmENZ/8yNvYN/VT4DdsqtRrZtMo0Og
PmZZIIpBTbUSyvg=
-----END CERTIFICATE-----

-----BEGIN ENCRYPTED PRIVATE KEY-----
MIGMMEAGCSqGSIb3DQEFDTAzMBsGCSqGSIb3DQEFDDAOBAg3P+ShojSyRAICCAAw
FAYIKoZIhvcNAwcECLhovJf8tmC3BEhjHrDcLeg8vPnIMXdl8j4J6fsmcoSV9d7V
MpXUYhi0SVVPA37O6LrLL6nqwRucIgZzVfpBKinmNvM9tdC4gVt+xepETiB1WsM=
-----END ENCRYPTED PRIVATE KEY-----
'''
PEM_PASS = 'BARS_PASS_sidurniq38ymlskjn,jh<HGjhBJKGjmHGJKGM'

class TestHelpersFunctions(unittest.TestCase):
    def setUp(self):
        self.empty_doc = etree.Element('Empty')

    def test_dict_to_xmldoc(self):
        test_dict = {
            'InfNode': {
                '__ns__': 'inf',
                'Var1': 'Hello',
                'Var2': 'world'
            },
            'NodeWithoutNs': {
                'Test1': 'Hello, world?',
                'Test2': 'Bandersnatch',
                'Surpise': {
                    '__ns__': 'smev',
                    'Recipient': 'Someone',
                    'Sender': 'Someone else'
                }
            }
        }

        dict_to_xmldoc(self.empty_doc, test_dict)

        paths_and_values = {
            '//Empty/InfNode/inf:Var1': test_dict['InfNode']['Var1'],
            '//Empty/InfNode/inf:Var2': test_dict['InfNode']['Var2'],
            '//Empty/NodeWithoutNs/Test1': test_dict['NodeWithoutNs']['Test1'],
            '//Empty/NodeWithoutNs/Test2': test_dict['NodeWithoutNs']['Test2'],
            '//Empty/NodeWithoutNs/Surpise/smev:Recipient': test_dict['NodeWithoutNs']['Surpise']['Recipient'],
            '//Empty/NodeWithoutNs/Surpise/smev:Sender': test_dict['NodeWithoutNs']['Surpise']['Sender'],
        }

        for xpath, val in paths_and_values.items():
            inf_node = self.empty_doc.xpath(xpath, namespaces=NS_MAP)
            assert inf_node, 'XPath "%s" returned no nodes!' % xpath
            self.assertEquals(inf_node[0].text, val)


class TestSkeletonFunction(unittest.TestCase):
    def setUp(self):
        self.ctx = {
            'Service': {
                'Mnemonic': 'MONR001001',
                'Version': '0.10'
            },
            'Sender': {
                'Code': "SEND01001",
                'Name': "Sender"
            },
            'Recipient': {
                'Code': 'RECV01001',
                'Name': 'Recipient'
            },
            'Originator': {
                'Code': "SEND01001",
                'Name': "Sender's origin"
            },
            'TypeCode': 'GSRV',
            'Status': 'REQUEST',
            'TestMsg': True
        }

        self.req = construct_smev_envelope('TestPacket', self.ctx)

    def test_envelope_creation(self):        
        prefix = './/SOAP-ENV:Body/inf:TestPacket/smev:Message'

        paths_and_values = {
            '%s/smev:Service/smev:Mnemonic' % prefix: self.ctx['Service']['Mnemonic'],
            '%s/smev:Service/smev:Version' % prefix: self.ctx['Service']['Version'],
            '%s/smev:Sender/smev:Code' % prefix: self.ctx['Sender']['Code'],
            '%s/smev:Sender/smev:Name' % prefix: self.ctx['Sender']['Name'],
            '%s/smev:Recipient/smev:Code' % prefix: self.ctx['Recipient']['Code'],
            '%s/smev:Recipient/smev:Name' % prefix: self.ctx['Recipient']['Name'],
            '%s/smev:Originator/smev:Code' % prefix: self.ctx['Originator']['Code'],
            '%s/smev:Originator/smev:Name' % prefix: self.ctx['Originator']['Name'],
            '%s/smev:TypeCode' % prefix: self.ctx['TypeCode'],
            '%s/smev:Status' % prefix: self.ctx['Status'],
            '%s/smev:TestMsg' % prefix: 'true',
        }

        for xpath, val in paths_and_values.items():
            node = self.req.xpath(xpath, namespaces=NS_MAP)
            assert node, 'XPath "%s" returned no nodes!' % xpath
            self.assertEquals(node[0].text, val)


class TestSigner(unittest.TestCase):
    def setUp(self):
        self.ctx = {
            'Service': {
                'Mnemonic': 'MONR001001',
                'Version': '0.10'
            },
            'Sender': {
                'Code': "SEND01001",
                'Name': "Sender"
            },
            'Recipient': {
                'Code': 'RECV01001',
                'Name': 'Recipient'
            },
            'Originator': {
                'Code': "SEND01001",
                'Name': "Sender's origin"
            },
            'TypeCode': 'GSRV',
            'Status': 'REQUEST',
            'TestMsg': True
        }

        self.req = construct_smev_envelope('TestPacket', self.ctx)
        self.tmp_file = NamedTemporaryFile(delete=False)
        self.tmp_file.write(PEM)
        self.tmp_file.close()

    def test_sign_verify(self):
        signed = sign_document(self.req, self.tmp_file.name, PEM_PASS)
        assert verify_envelope_signature(signed), 'Signing mechanism produced invalid signature :('

        sender_node = signed.xpath('.//SOAP-ENV:Body/inf:TestPacket/smev:Message/smev:Sender/smev:Name', namespaces=NS_MAP)[0]
        sender_node.text = 'Impersonator'
        assert not verify_envelope_signature(signed), 'Document was changed, but signature still verifies!'


    def tearDown(self):
        os.remove(self.tmp_file.name)


class TestAttachments(unittest.TestCase):
    def setUp(self):
        self.directory = mkdtemp()
        self.files = ['%s%s' % (str(uuid.uuid4()), ext) for ext in types_map.keys()]

        # Файл без расширения, по которому можно было бы определить MIME-тип        
        
        self.example_text = str(uuid.uuid4())
        self.example_hash = get_text_digest(self.example_text)
        self.example_sig_hash = get_text_digest(self.example_hash)

        for fn in self.files:
            with open(os.path.join(self.directory, fn), 'w') as f:
                f.write(self.example_text)

    def testEncode(self):        
        req_code, encoded_zip = encode_directory(self.directory)
        decoded_zip = base64.b64decode(encoded_zip)
        sio = StringIO.StringIO(decoded_zip)
        zip_arc = zipfile.ZipFile(sio, 'r')

        for fn in self.files:
            f = zip_arc.open(fn, 'r')
            text = f.read()
            f.close()

            sig_f = zip_arc.open('%s.sig' % fn, 'r')
            dgst_text = sig_f.read()
            sig_f.close()

            assert dgst_text == self.example_hash, u'Digests are not identical!'
            assert text == self.example_text, u'Text in file is corrupted!'

        with open('tmp.zip', 'wb') as f:
            f.write(decoded_zip)

    def testExtract(self):
        req_code, encoded_zip = encode_directory(self.directory)
        manifest, extracted_to = extract_directory(req_code, encoded_zip)

        applied_documents_node = manifest.xpath('//AppliedDocuments')
        assert applied_documents_node, '"AppliedDocuments" node not found!'
        applied_documents = applied_documents_node[0].xpath('.//AppliedDocument')
        assert applied_documents, 'No documents in manifest!'

        for doc in applied_documents:
            doc_info = dict([(n.tag, n.text) for n in doc])

            if doc_info['URL'].endswith('.sig'):
                assert doc_info['DigestValue'] == self.example_sig_hash, u'Digests are not identical!'
            else:
                assert doc_info['URL'] in self.files, u'Path "%s" outside of testing set!' % doc_info['URL']
                assert doc_info['DigestValue'] == self.example_hash, u'Digests are not identical!'

            dot_pos = doc_info['URL'].rindex('.')
            if dot_pos:
                ext = doc_info['URL'][dot_pos:]
                if ext != '.sig':
                    assert doc_info['Type'] == types_map[ext], 'Invalid MIME type "%s" for "%s"' % (doc_info['Type'], ext)
            else:
                assert doc_info['Type'] == 'application/octet-stream', 'File without extension not classified as octet-stream!'

    def tearDown(self):
        shutil.rmtree(self.directory)

if __name__ == '__main__':
    unittest.main()



#coding: utf-8

import base64
import os
import tempfile
import uuid
from zipfile import ZipFile
from StringIO import StringIO
from mimetypes import types_map as mime_types_map
from lxml import etree

from signer import get_file_digest, get_text_digest
from helpers import make_node, dict_to_xmldoc, parse_xml_string


class InvalidManifestException(Exception):
    pass


class InvalidFileDigestException(Exception):
    pass


def encode_directory(directory):
    u'''
    Преобразование содержимого папки и её структуры в вид, пригодный для присоединения
    к СМЭВ-сообщению согласно МР 2.4.4-2.5.6. 

    Результатом выполнения будет кортеж с уникальным GUID кодом (поле заголовка RequestCode)
    и закодированный в base64 ZIP-архив с манифестом, файлами директории и соответствующими
    файлами подписей.

    ZIP-архив формируется в памяти.

    @param  directory   Путь к папке, содержимое которой необходимо прикрепить.
    @type   directory   unicode

    @return GUID и закодированный в base64 ZIP-архив.
    @rtype  (unicode, unicode)    
    '''
    # Генерируем код запроса
    request_code = str(uuid.uuid4())
    i = 1

    in_memory_file = StringIO()
    zip_arc = ZipFile(in_memory_file, 'w')

    applied_documents_node = make_node('AppliedDocuments')
    for (path, subdirs, files) in os.walk(directory):
        for fn in files:            
            path_to_file = os.path.join(path, fn).replace('\\', '/').replace('\\\\', '/')
            relative_path = path_to_file[len(directory):].lstrip('/')
            dgst = get_file_digest(path_to_file)
            dot_pos = fn.find('.')

            applied_documents = [
                {
                    'URL': relative_path,
                    'Name': fn,
                    'DigestValue': dgst,
                    # Пытаемся определить MIME-тип файла, но если нет - бинарный файл.
                    'Type': mime_types_map.get(fn[dot_pos:], 'application/octet-stream'),
                    # TODO: выяснить правила генерации кода документа
                    'CodeDocument': u'0000',
                    'Number': i,
                },
                # Файл с подписью по PKCS/7
                {
                    'URL': u'%s%s' % (relative_path, '.sig'),
                    'Name': u'%s.sig' % fn,
                    'DigestValue': get_text_digest(dgst),
                    'Type': 'application/x-pkcs7-signature',
                    'CodeDocument': u'0000',
                    'Number': i + 1,
                }]

            i += 2

            for doc in applied_documents:
                app_doc_node = make_node('AppliedDocument')
                dict_to_xmldoc(app_doc_node, doc)
                applied_documents_node.append(app_doc_node)

            # Добавляем в ZIP-архив файл и его подпись
            zip_arc.write(path_to_file, arcname=relative_path)
            zip_arc.writestr('%s.sig' % relative_path, dgst)

    # Добавляем в ZIP-архив манифест и его подпись
    manifest_str = etree.tostring(applied_documents_node, pretty_print=True)
    zip_arc.writestr('req_%s.xml' % request_code, manifest_str.encode('utf-8'))
    zip_arc.writestr('req_%s.sig' % request_code, get_text_digest(manifest_str))

    zip_arc.close()
    in_memory_file.seek(0)

    # Преобразуем ZIP-архив в base64
    encoded = base64.b64encode(in_memory_file.read())
    in_memory_file.close()

    return request_code, encoded


def extract_directory(request_code, binary_data, destination=None,
                      verify=True, exclude_sigs=True):
    u'''
    Извлечение файлов из закодированного по МР архива вложений.
    '''

    # Распаковываем архив
    decoded = base64.b64decode(binary_data)
    in_memory_file = StringIO(decoded)
    zip_arc = ZipFile(in_memory_file, 'r')

    # Пробуем получить манифест из архива
    try:
        manifest_file = zip_arc.open('req_%s.xml' % request_code, 'r')
        manifest_str = manifest_file.read()
        manifest_file.close()
    except KeyError:
        raise InvalidManifestException(u'Manifest file "req_%s.xml" not found' % request_code)

    manifest = parse_xml_string(manifest_str)
    applied_documents = manifest.xpath('.//AppliedDocument')

    if not destination:
        destination = tempfile.mkdtemp()

    for doc in applied_documents:
        doc_info = dict([(n.tag, n.text) for n in doc])

        # Мы проверяем подписи по манифесту, поэтому по умолчанию
        # игнорируем файлы с ними и не распаковываем
        if doc_info['Name'].endswith('.sig') and exclude_sigs:
            continue

        zip_arc.extract(doc_info['URL'], destination)
        path_to_file = os.path.join(destination, doc_info['URL'])

        # Проверяем подписи файлов по данным из манифеста
        if verify:
            dgst = get_file_digest(path_to_file)
            if doc_info['DigestValue'] != dgst:
                raise InvalidFileDigestException((doc_info['URL'],
                                                 doc_info['DigestValue'],
                                                 dgst))

    zip_arc.close()
    in_memory_file.close()

    return manifest, destination



#coding: utf-8

import subprocess

from lxml import etree
from lxml.etree import XMLSyntaxError

from namespaces import NS_MAP, REVERSE_NS_MAP, make_node_with_ns


class Fault(Exception):
    u'''
    Системная ошибка.
    '''
    pass


def tags(doc, path, ns_map=None):
    u'''
    Выборка XML-элемента по указанному XPath c заранее прописанным
    пространством имен.

    Существует исключительно для удобства.

    @param  doc     XML-документ, по которому производится выборка.
    @type   doc     lxml.Element

    @param  path    XPath-выражение.
    @type   path    unicode

    @param  ns_map  Словарь с пространством имен.
    @type   dict

    @return Найденные по выражению тэги.
    @rtype  list of lxml.Element
    '''

    if ns_map is None:
        ns_map = NS_MAP
    return doc.xpath(path, namespaces=ns_map)


def tag_single(doc, path, ns_map=None):
    u'''
    Выбор единичного XML-элемента по указанному XPath c заранее прописанным
    пространством имен.

    Если элемент не найден, возвращается None.
    Если найден более чем один элемент - выбрасывается исключение ValueError.

    Существует исключительно для удобства.

    @param  doc     XML-документ, по которому производится выборка.
    @type   doc     lxml.Element

    @param  path    XPath-выражение.
    @type   path    unicode

    @param  ns_map  Словарь с пространством имен.
    @type   ns_map  dict

    @return Найденный XML-элемент.
    @rtype  lxml.Element
    '''

    result = tags(doc, path, ns_map)
    if not result:
        return None
    if len(result) > 1:
        raise ValueError('Multiple tags found for "%s"' % path)
    return result[0]


# Замыкание для создания XML-элемента с указанным именем.
make_node = lambda el_name: etree.Element(('%s' % el_name))


def run_cmd(cmd, input=None):
    u'''
    Выполняем команду в интерпретаторе ОС и считываем выводимые ею данные.

    @param  cmd     Команда для запуска.
    @type   cmd     unicode

    @param  input   Входные данные, которые будут поданы на stdin.
    @type   input   unicode

    @return Вывод вызванной программы
    @rtype  unicode
    '''
    pr = subprocess.Popen(
        cmd,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)

    return pr.communicate(input=input)


def parse_xml_string(xml_string, charset=u'utf-8',
                     parser=etree.XMLParser(remove_comments=True)):
    u'''
    Разбор строки, содержащей XML-документ с настраиваемым парсером.

    По умолчанию используется парсер, удаляющий комментарии из документа.

    @param  xml_string  Строка, содержащая XML-документ.
    @type   xml_string  unicode

    @param  charset     Кодировка.
    @type   charset     unicode

    @param  parser      Парсер, используемый для разбора строки.
    @type   parser      function       

    @return Корень XML-документа.
    @rtype  lxml.Element
    '''
    try:
        try:
            root, xmlids = etree.XMLID(xml_string, parser)
        except XMLSyntaxError as err:
            raise Fault(unicode(err))
    except ValueError:
        try:
            root, xmlids = etree.XMLID(xml_string.encode(charset), parser)
        except XMLSyntaxError as err:
            raise Fault(unicode(err))
    return root


def _from_soap(envelope):
    u'''
    Выделение заголовка и тела SOAP-запроса из XML-документа.

    @param  envelope    XML-документ, содержащий SOAP-запрос.
    @type   envelope    lxml.Element

    @return Заголовок и тело запроса.
    @rtype  list of lxml.Element
    '''

    # Выделение из конверта заголовка и тела запроса.
    if envelope.tag != "{%s}Envelope" % NS_MAP['SOAP-ENV']:
        raise Fault("No {%s}Envelope element was found!" % NS_MAP['SOAP-ENV'])

    header_envelope = envelope.xpath("e:Header", namespaces={"e": NS_MAP['SOAP-ENV']})
    body_envelope = envelope.xpath("e:Body", namespaces={"e": NS_MAP['SOAP-ENV']})

    if len(header_envelope) == 0 and len(body_envelope) == 0:
        raise Fault("Soap envelope is empty!")

    header = None
    if len(header_envelope) > 0:
        header = header_envelope[0]

    body = None
    if len(body_envelope) > 0:
        body = body_envelope[0]

    return header, body


def extract_smev_parts(envelope):
    u'''
    Выделение из SOAP-запроса относящихся к СМЭВу частей.

    @param  envelope    XML-документ, содержащий SOAP-запрос.
    @type   envelope    lxml.Element

    @return Информация о подписи, заголовок сообщения, сообщение.
    @rtype  list of lxml.Element
    '''

    header, body = _from_soap(envelope)

    security_node = tags(header, 'wsse:Security')
    if security_node:
        security_node = security_node[0]
        token_node = tags(security_node, 'wsse:BinarySecurityToken')[0]
        signature_node = tags(security_node, 'ds:signature_node')[0]
    else:
        token_node = signature_node = None

    message_node = tags(body, '//smev:Message')[0]
    message_data_node = tags(body, '//smev:MessageData')[0]

    return token_node, signature_node, message_node, message_data_node


def dict_to_xmldoc(node, d, inherited_ns=None):
    u'''
    Преобразование питоновского словаря в структурированное дерево
    XML-элементов с поддержкой выставления пространства имен.

    @param  node    XML-элемент, к которому будут прикреплены созданные
                    элементы.
    @type   node    lxml.Element

    @param  d   Преобразуемый словарь.
    @type   d   dict
    '''

    ns = d.get('__ns__', inherited_ns)

    node_factory = make_node_with_ns(ns) if ns else make_node

    for k, v in d.iteritems():
        if k == '__ns__':
            continue

        sub_node = node_factory(k)
        if isinstance(v, list):
            for el in v:
                el_node = make_node(k)
                dict_to_xmldoc(el_node, el, inherited_ns=ns)
                sub_node.append(el_node)
        elif isinstance(v, dict):
            dict_to_xmldoc(sub_node, v, inherited_ns=ns)
        else:
            sub_node.text = unicode(v)
        node.append(sub_node)


def xmldoc_to_dict(node, include_ns=True, ns_map=REVERSE_NS_MAP):
    u'''
    Преобразование XML-элемента и всех подчиненных ему в древовидную
    структуру внутри питоновского словаря. Пространства имен автоматически
    определяются по переданной карте соответствий, где ключами являются
    полноценные URL-ы к схемам, а значениями - кодовые имена пространств имен.

    Если среди дочерних элементов узла встречаются 2 или более с одинаковыми
    названиями, то в ключе будет храниться список из таких узлов.

    Обратно совместим с dict_to_xmldoc.

    @param  node    XML-элемент, который преобразуется в словарь.
    @type   node    lxml.Element

    @param  include_ns  Флаг сокрытия пространств имен элементов из конечного
                        результата.
    @type   include_ns  boolean

    @param  ns_map      Карта соответсвия пространств имен кодовым идентификаторам.
    @type   ns_map      dict вида {'http://schemas.xmlsoap.org/soap/envelope': 'SOAP-ENV'}
    '''

    def get_ns(n):
        if n.tag[0] == '{':
            ns, tag = n.tag[1:].split('}', 1)
            return ns
        return ''

    def get_tag(n):
        if n.tag[0] == '{':
            ns, tag = n.tag.split('}', 1)
            return tag
        return n.tag

    def flatten(d):
        res = {}

        for k, v in d.iteritems():
            if isinstance(v, list) and len(v) == 1:
                res[k] = v[0]
            else:
                res[k] = v
        return res

    result = {}

    ns = get_ns(node)
    if include_ns and ns:
        result['__ns__'] = REVERSE_NS_MAP.get(ns, ns)

    if len(node):
        for child in node:
            result.setdefault(get_tag(child), []).append(
                xmldoc_to_dict(child, include_ns=include_ns))
        return flatten(result)
    else:
        return node.text



#coding: utf-8

import json
import requests


def portal_check_flk(message, version='2.5.6'):
    versions = {
        '2.4.4': 'rev111111',
        '2.4.5': 'rev111111_2.4.5',
        '2.5.5': 'rev120315',
        '2.5.6': 'rev120315_2.5.6'
    }

    assert version in versions, u'Unknown method version: %s' % version
    assert message, u'Message not specified!'

    req = {
        'message': message,
        'configId': versions[version]
    }

    r = requests.post('http://smev.gosuslugi.ru/portal/api/services/flcApply',
                      data=json.dumps(req),
                      proxies=dict(http=''),
                      headers={'content-type': 'application/json'})
    r.raise_for_status()

    reply = json.loads(r.content)['error']
    return reply['errorCode'] == 0, (reply['errorMessage'] or '').split('\n')


def portal_check_signature(message):
    assert message, u'Message not specified!'

    req = {
        'message': message,
        'actor': 'http://smev.gosuslugi.ru/actors/smev'
    }

    r = requests.post('http://smev.gosuslugi.ru/portal/api/services/verifySignature',
                      data=json.dumps(req),
                      proxies=dict(http=''),
                      headers={'content-type': 'application/json'})
    r.raise_for_status()

    reply = json.loads(r.content)['error']
    return reply['errorCode'] == 0, (reply['errorMessage'] or '').split('\n')



#coding: utf-8

from lxml import etree

NS_MAP = {
    "ds": "http://www.w3.org/2000/09/xmldsig#",
    "smev": "http://smev.gosuslugi.ru/rev120315",
    "SOAP-ENV": "http://schemas.xmlsoap.org/soap/envelope/",
    "wsse": "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd",
    "wsu": "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd",
    "ws": "http://pe.minregion.ru/",
    "inf": "http://smev.gosuslugi.ru/inf/"
}

REVERSE_NS_MAP = dict([(v, k) for k, v in NS_MAP.items()])

make_node_with_ns = lambda ns: lambda el_name: etree.Element('{%s}%s' % (NS_MAP[ns], el_name), nsmap={ns: NS_MAP[ns]})



#coding: utf-8

import copy

from lxml import etree
from datetime import datetime

from helpers import make_node, extract_smev_parts, tag_single, dict_to_xmldoc
from namespaces import NS_MAP, make_node_with_ns


SMEV_VERSIONS = ['2.4.4', '2.5.5', '2.5.6']


# Исключение, выбрасываемое в случае отсутствия возможности
# конвертации между указанными версиями СМЭВ
class NoViableConversionError(Exception): pass


def convert_smev_request(envelope, from_ver='2.5.6', to_ver=None):
    u'''
    Экспериментальный механизм преобразования структуры сообщений согласно
    предписаний из различных версий методических рекомендаций.

    В случае отсутствия пути конвертации между указанными версиями выбрасывается
    исключение.

    Внимание: преобразование происходит прямо над переданным объектом,
    _не_ над копией.

    @param  envelope    Преобразуемое СМЭВ сообщение в виде дерева XML.
    @type   envelope    lxml.Element

    @param  from_ver    Версия переданного сообщения.
    @type   from_ver    unicode

    @param  from_ver    Версия, в которую необходимо преобразовать сообщение.
    @type   from_ver    unicode

    @return Преобразованное СМЭВ-сообщение.
    @rtype  lxml.Element
    '''

    smev_node = make_node_with_ns('smev')
    token, signature, msg_node, msg_data_node = extract_smev_parts(envelope)

    def convert_256_to_255(envelope):
        service_node = msg_data_node.xpath('//smev:Service', namespaces=NS_MAP)[0]
        name_node = msg_data_node.xpath('//smev:Service/smev:Mnemonic', namespaces=NS_MAP)
        if name_node:
            name = name_node[0].text
        servicename_node = smev_node('ServiceName')
        servicename_node.text = name
        service_node.getparent().remove(service_node)
        msg_node.insert(3,  servicename_node)

        return envelope

    mapping = {
        ('2.5.6', '2.5.5'): convert_256_to_255
    }

    if not (from_ver, to_ver) in mapping:
        raise NoViableConversionError("from %s to %s" % (from_ver, to_ver))

    return mapping[(from_ver, to_ver)](envelope)


def create_empty_context(version='2.5.6'):
    u'''
    Создание пустого контекста запроса, используемого для формирования 
    СМЭВ-сообщения. В зависимости от версии, набор необходимых полей может 
    меняться. По умолчанию формируется контекст для создания сообщения
    по версии 2.5.6 МР и проставляется флаг тестового взаимодействия.

    В случае запроса формирования контекста по неподдерживаемой нами
    версии МР - выьрасывается исключение.
    '''

    blanks = {
        '2.5.6': {
            'Sender': {
                'Code': u'',
                'Name': u''
            },
            'Originator': {
                'Code': u'',
                'Name': u''
            },
            'Recipient': {
                'Code': u'',
                'Name': u'Recipient'
            },
            'Service': {
                'Mnemonic': u'',
                'Version': u''
            },
            'Status': 'REQUEST',
            'TypeCode': 'GSRV',
            'TestMsg': True
        },
        '2.5.5': {
            'Sender': {
                'Code': u'',
                'Name': u''
            },
            'Originator': {
                'Code': u'',
                'Name': u''
            },
            'Recipient': {
                'Code': u'',
                'Name': u'Recipient'
            },
            'ServiceName': u'',
            'Status': 'REQUEST',
            'TypeCode': 'GSRV',
            'TestMsg': True
        }
    }

    assert version in blanks, 'Unknown SMEV version: %s' % version
    return blanks[version]


def extract_context_from_envelope(envelope):
    u'''
    Формирование контекста на основе данных из существующего сообщения СМЭВ.
    На данный момент поддерживается только обработка структуры сообщений по 
    МР версии 2.5.6.

    Если какой-либо из элементов содержит текст "true" или "false", то значение
    будет заменено на True или False соответственно.

    @param  envelope    Сообщение СМЭВ.
    @type   envelope    lxml.Element

    @return Словарь контекста.
    @rtype  dict

    '''

    smev_version = '2.5.6'

    ctx = create_empty_context(version=smev_version)
    ctx['Sender']['Code'] = tag_single(envelope, './/smev:Sender/smev:Code')
    ctx['Sender']['Name'] = tag_single(envelope, './/smev:Sender/smev:Name')

    ctx['Originator']['Code'] = tag_single(envelope, './/smev:Originator/smev:Code')
    ctx['Originator']['Name'] = tag_single(envelope, './/smev:Originator/smev:Name')

    ctx['Recipient']['Code'] = tag_single(envelope, './/smev:Recipient/smev:Code')
    ctx['Recipient']['Name'] = tag_single(envelope, './/smev:Recipient/smev:Name')

    ctx['Service']['Mnemonic'] = tag_single(envelope, './/smev:Service/smev:Mnemonic')
    ctx['Service']['Version'] = tag_single(envelope, './/smev:Service/smev:Version')

    ctx['Status'] = tag_single(envelope, './/smev:Status')
    ctx['TypeCode'] = tag_single(envelope, './/smev:TypeCode')
    ctx['TestMsg'] = tag_single(envelope, './/smev:TestMsg')

    def format_tag_contents(root):
        for k, v in root.iteritems():
            if isinstance(v, dict):
                root[k] = format_tag_contents(v)
            else:
                root[k] = getattr(v, 'text', v)
                if root[k] in ('true', 'false'):
                    root[k] = root[k] == 'true'
        return root

    return format_tag_contents(ctx)


def construct_smev_envelope(action_name, context, nsmap=None, version='2.5.6'):
    u'''
    Составления обертки СМЭВ-сообщения на основе переданного контекста и имени
    блока с данными.

    @param  action_name     Имя блока, содержащего данные сообщения.
    @type   action_name     unicode

    @param  context     Словарь с данными заголовка СМЭВ-сообщения.
    @type   context     dict

    @param  nsmap   Карта пространств имен XML-документа.
    @type   nsmap   dict

    @param  version     Версия методических рекомендаций, используемая при 
                        создании обертки сообщения.
    @type   version     unicode

    @return Созданное СМЭВ-сообщение.
    @rtype  lxml.Element
    '''

    required_fields = [
        'Sender',
        'Recipient',
        'TypeCode',
        'Status',
    ]
    type_codes = (
        'GSRV',  # Оказание государственных услуг
        'GFNC',  # Исполнение государственных функций
        'OTHR'   # Взаимодействие в иных целях
    )
    statuses = (
        'ACCEPT',  # Сообщение-квиток о приеме
        'CANCEL',  # Отзыв заявления
        'FAILURE',  # Технический сбой
        'INVALID',  # Ошибка при ФЛК (форматно-логический контроль)
        'NOTIFY',  # Уведомление об ошибке
        'PING',  # Запрос данных/результатов
        'PACKET',  # Пакетный режим обмена
        'PROCESS',  # В обработке
        'REJECT',  # Мотивированный отказ
        'REQUEST',
        'RESULT',
        'STATE'  # Возврат состояния
    )

    for name in required_fields:
        assert name in context, u'Required field "%s" missing from context!' % name
    assert context['TypeCode'] in type_codes, u'Type code should be one of %s' % (type_codes,)
    assert context['Status'] in statuses, u'Status code should be one of %s' % (statuses,)

    _ns_map = copy.copy(NS_MAP)
    if nsmap:
        _ns_map.update(nsmap)

    envelope = etree.Element("{%s}Envelope" % _ns_map['SOAP-ENV'], nsmap=_ns_map)
    header_node = etree.Element("{%s}Header" % _ns_map['SOAP-ENV'])
    body_node = etree.Element("{%s}Body" % _ns_map['SOAP-ENV'], attrib={"{%s}Id" % _ns_map['wsu']: "body"})

    message_node = etree.Element("{%s}Message" % _ns_map['smev'])

    # Данные о системе-инициаторе взаимодействия (Поставщике)
    sender_node = etree.Element("{%s}Sender" % _ns_map['smev'])
    sender_code_node = etree.Element("{%s}Code" % _ns_map['smev'])
    sender_name_node = etree.Element("{%s}Name" % _ns_map['smev'])
    sender_code_node.text = context['Sender']['Code']
    sender_name_node.text = context['Sender']['Name']

    # Данные о системе-получателе сообщения (Потребителе)
    recipient_node = etree.Element("{%s}Recipient" % _ns_map['smev'])
    recipient_code_node = etree.Element("{%s}Code" % _ns_map['smev'])
    recipient_name_node = etree.Element("{%s}Name" % _ns_map['smev'])
    recipient_code_node.text = context['Recipient']['Code']
    recipient_name_node.text = context['Recipient']['Name']

    # Данные о системе, инициировавшей цепочку из нескольких
    # запросов-ответов, объединенных единым процессом в рамках
    # взаимодействия
    originator_node = etree.Element("{%s}Originator" % _ns_map['smev'])
    originator_code_node = etree.Element("{%s}Code" % _ns_map['smev'])
    originator_name_node = etree.Element("{%s}Name" % _ns_map['smev'])
    originator_code_node.text = context['Originator']['Code']
    originator_name_node.text = context['Originator']['Name']

    # Данные о вызванном сервисе
    if version == '2.5.6':
        service_node = etree.Element("{%s}Service" % _ns_map['smev'])
        service_mnemonic_node = etree.Element("{%s}Mnemonic" % _ns_map['smev'])
        service_version_node = etree.Element("{%s}Version" % _ns_map['smev'])
        service_mnemonic_node.text = context['Service']['Mnemonic']
        service_version_node.text = context['Service']['Version']
        
        service_node.extend([
            service_mnemonic_node,
            service_version_node])
    else:
        service_node = etree.Element("{%s}ServiceName" % _ns_map['smev'])
        service_node.text = context['ServiceName']

    # Тип сообщения по классификатору типов сообщений,  передаваемых через
    # узел СМЭВ (приложение 2 метод. рекомендаций )
    typecode_node = etree.Element("{%s}TypeCode" % _ns_map['smev'])
    typecode_node.text = context.get('TypeCode', '')

    # Сведения о статусе электронного сообщения (см. приложение 2)
    status_node = etree.Element("{%s}Status" % _ns_map['smev'])
    status_node.text = context.get('Status', '')

    # Дата и время создания сообщения в формате UTC
    # 'yyyy-MM-dd'T'HH:mm:ss.SSSZ’
    utc_now = datetime.strftime(datetime.utcnow(), "%Y-%m-%dT%H:%M:%S.%f")[:-2]
    date_node = etree.Element("{%s}Date" % _ns_map['smev'])
    date_node.text = context.get('Date', utc_now)

    # Признак принадлежности электронного сообщения различным категориям
    # взаимодействия, возникающим при межведомственном обмене (приложение 2)
    # По умолчанию выставляется "Неопределенная категория"
    exchangetype_node = etree.Element("{%s}ExchangeType" % _ns_map['smev'])
    exchangetype_node.text = context.get('Exchangetype', '0')

    # Признак тестового режима
    testmsg_node = None
    if context.get('TestMsg', True):
        testmsg_node = etree.Element("{%s}TestMsg" % _ns_map['smev'])
        testmsg_node.text = 'true'

    messagedata_node = etree.Element("{%s}MessageData" % _ns_map['smev'])
    appdata_node = etree.Element("{%s}AppData" % _ns_map['smev'], attrib={"{%s}Id" % _ns_map['wsu']: "AppData"})
    app_document_node = etree.Element("{%s}AppDocument" % _ns_map['smev'])
    binarydata_node = etree.Element("{%s}BinaryData" % _ns_map['smev'])
    requestcode_node = etree.Element("{%s}RequestCode" % _ns_map['smev'])

    app_document = context.get('AppDocument')
    if app_document is not None:
        if isinstance(app_document, dict):
            requestcode_node.text = app_document['RequestCode'] or ''
            binarydata_node.text = app_document['BinaryData'] or ''
        else:
            app_document_node.text = app_document or ''

    app_document_node.extend([
        requestcode_node,
        binarydata_node])

    messagedata_node.extend([
        appdata_node,
        app_document_node])

    sender_node.extend([
        sender_code_node,
        sender_name_node])

    recipient_node.extend([
        recipient_code_node,
        recipient_name_node])

    originator_node.extend([
        originator_code_node,
        originator_name_node])

    message_node.extend([
        sender_node,
        recipient_node,
        originator_node,
        service_node,
        typecode_node,
        status_node,
        date_node,
        exchangetype_node,
    ])
    if testmsg_node is not None:
        message_node.append(testmsg_node)

    if 'CaseNumber' in context:
        # Номер дела в ИС отправителя
        casenumber_node = etree.Element("{%s}CaseNumber" % _ns_map['smev'])
        casenumber_node.text = context['CaseNumber']
        message_node.append(casenumber_node)

    own_section_node = etree.Element("{%s}%s" % (_ns_map['inf'], action_name), nsmap=_ns_map)
    own_section_node.extend([
        message_node,
        messagedata_node])

    body_node.append(
        own_section_node)
    envelope.extend([
        header_node,
        body_node])

    return envelope


def construct_error_reply(original_req, err_code, msg, custom_status=None):
    u'''
    Создание ответ на СМЭВ-сообщение, который будет содержать в себе 
    код и сообщение об ошибке.

    @param  original_req    СМЭВ-сообщение, на которое формируется ответ.
    @type   original_req    lxml.Element

    @param  err_code    Код сообщения об ошибке.
    @type   err_code    unicode

    @param  msg    Текст сообщения об ошибке.
    @type   msg    unicode

    @param  custom_status    Статус в заголовке СМЭВ-сообщения.
    @type   custom_status    unicode

    @return Ответное сообщение об ошибке.
    @rtype  lxml.Element
    '''

    reply_ctx = extract_context_from_envelope(original_req)

    recipient_code = reply_ctx['Recipient']['Code']
    recipient_name = reply_ctx['Recipient']['Name']

    reply_ctx['Recipient']['Code'] = reply_ctx['Sender']['Code']
    reply_ctx['Recipient']['Name'] = reply_ctx['Sender']['Name']

    reply_ctx['Sender']['Code'] = recipient_code
    reply_ctx['Sender']['Name'] = recipient_name

    reply_ctx['Status'] = custom_status or 'REJECT'

    reply_req = construct_smev_envelope('Error', reply_ctx)
    appdata_node = tag_single(reply_req, './/smev:AppData')

    dict_to_xmldoc(appdata_node, {
                   '__ns__': 'inf', 
                   'Error': dict(errorCode=err_code, errorMessage=msg)})

    return reply_req



#coding: utf-8

import os
from setuptools import setup


def read(fn):
    return open(os.path.join(os.path.dirname(__file__), fn)).read()

setup(
    name='m3-libsmev',
    version='0.1.5',
    author='Borisov Kirill',
    author_email='borisov@bars-open.ru',
    description=("Library of low-level helpers that make integration with SMEV "
                 "somewhat less painful"),
    license="MIT",
    keywords="lxml smev m3 bars",
    long_description=read('README.rst'),
    packages=['libsmev'],
    install_requires=['libxml2-python >= 2.6.9', 'lxml >= 3.1.0', 'requests >= 1.2.0'],
    classifiers=(
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 5 - Production/Stable',
        )
)
